@@ -3,7 +3,8 @@
 Open Asset Import Library (assimp)
 ---------------------------------------------------------------------------
 
-Copyright (c) 2006-2015, assimp team
+Copyright (c) 2006-2017, assimp team
+
 
 All rights reserved.
 
@@ -46,7 +47,7 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 #include "PretransformVertices.h"
 #include "ProcessHelper.h"
-#include "SceneCombiner.h"
+#include <assimp/SceneCombiner.h>
 #include "Exceptional.h"
 
 using namespace Assimp;
@@ -397,7 +398,7 @@ void PretransformVertices::BuildWCSMeshes(std::vector<aiMesh*>& out, aiMesh** in
 
                 out.push_back(ntz);
 
-                node->mMeshes[i] = numIn + out.size() - 1;
+                node->mMeshes[i] = static_cast<unsigned int>(numIn + out.size() - 1);
             }
         }
     }
@@ -483,7 +484,7 @@ void PretransformVertices::Execute( aiScene* pScene)
             memcpy(npp,pScene->mMeshes,sizeof(aiMesh*)*pScene->mNumMeshes);
             memcpy(npp+pScene->mNumMeshes,&apcOutMeshes[0],sizeof(aiMesh*)*apcOutMeshes.size());
 
-            pScene->mNumMeshes  += apcOutMeshes.size();
+            pScene->mNumMeshes  += static_cast<unsigned int>(apcOutMeshes.size());
             delete[] pScene->mMeshes; pScene->mMeshes = npp;
         }
 
@@ -618,6 +619,7 @@ void PretransformVertices::Execute( aiScene* pScene)
         // transformation of the corresponding node
         l->mPosition   = nd->mTransformation * l->mPosition;
         l->mDirection  = aiMatrix3x3( nd->mTransformation ) * l->mDirection;
+        l->mUp         = aiMatrix3x3( nd->mTransformation ) * l->mUp;
     }
 
     if( !configKeepHierarchy ) {
@@ -644,7 +646,7 @@ void PretransformVertices::Execute( aiScene* pScene)
             {
                 aiNode* pcNode = *nodes = new aiNode();
                 pcNode->mParent = pScene->mRootNode;
-                pcNode->mName.length = ::sprintf(pcNode->mName.data,"mesh_%u",i);
+                pcNode->mName.length = ::ai_snprintf(pcNode->mName.data,MAXLEN,"mesh_%u",i);
 
                 // setup mesh indices
                 pcNode->mNumMeshes = 1;
@@ -656,7 +658,7 @@ void PretransformVertices::Execute( aiScene* pScene)
             {
                 aiNode* pcNode = *nodes = new aiNode();
                 pcNode->mParent = pScene->mRootNode;
-                pcNode->mName.length = ::sprintf(pcNode->mName.data,"light_%u",i);
+                pcNode->mName.length = ai_snprintf(pcNode->mName.data, MAXLEN, "light_%u",i);
                 pScene->mLights[i]->mName = pcNode->mName;
             }
             // generate camera nodes
@@ -664,7 +666,7 @@ void PretransformVertices::Execute( aiScene* pScene)
             {
                 aiNode* pcNode = *nodes = new aiNode();
                 pcNode->mParent = pScene->mRootNode;
-                pcNode->mName.length = ::sprintf(pcNode->mName.data,"cam_%u",i);
+                pcNode->mName.length = ::ai_snprintf(pcNode->mName.data,MAXLEN,"cam_%u",i);
                 pScene->mCameras[i]->mName = pcNode->mName;
             }
         }
@@ -689,9 +691,9 @@ void PretransformVertices::Execute( aiScene* pScene)
 
         // find the dominant axis
         aiVector3D d = max-min;
-        const float div = std::max(d.x,std::max(d.y,d.z))*0.5f;
+        const ai_real div = std::max(d.x,std::max(d.y,d.z))*ai_real( 0.5);
 
-        d = min+d*0.5f;
+        d = min + d * (ai_real)0.5;
         for (unsigned int a = 0; a <  pScene->mNumMeshes; ++a) {
             aiMesh* m = pScene->mMeshes[a];
             for (unsigned int i = 0; i < m->mNumVertices;++i) {
@@ -707,17 +709,16 @@ void PretransformVertices::Execute( aiScene* pScene)
 
         DefaultLogger::get()->debug("PretransformVerticesProcess finished");
 
-        sprintf(buffer,"Removed %u nodes and %u animation channels (%u output nodes)",
+        ::ai_snprintf(buffer,4096,"Removed %u nodes and %u animation channels (%u output nodes)",
             iOldNodes,iOldAnimationChannels,CountNodes(pScene->mRootNode));
         DefaultLogger::get()->info(buffer);
 
-        sprintf(buffer,"Kept %u lights and %u cameras",
+        ai_snprintf(buffer, 4096,"Kept %u lights and %u cameras",
             pScene->mNumLights,pScene->mNumCameras);
         DefaultLogger::get()->info(buffer);
 
-        sprintf(buffer,"Moved %u meshes to WCS (number of output meshes: %u)",
+        ai_snprintf(buffer, 4096,"Moved %u meshes to WCS (number of output meshes: %u)",
             iOldMeshes,pScene->mNumMeshes);
         DefaultLogger::get()->info(buffer);
     }
 }
-
