@@ -3,7 +3,8 @@
 Open Asset Import Library (assimp)
 ---------------------------------------------------------------------------
 
-Copyright (c) 2006-2015, assimp team
+Copyright (c) 2006-2017, assimp team
+
 
 All rights reserved.
 
@@ -50,11 +51,11 @@ OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "RawLoader.h"
 #include "ParsingUtils.h"
 #include "fast_atof.h"
-#include <boost/scoped_ptr.hpp>
-#include "../include/assimp/IOSystem.hpp"
-#include "../include/assimp/DefaultLogger.hpp"
-#include "../include/assimp/scene.h"
-
+#include <memory>
+#include <assimp/IOSystem.hpp>
+#include <assimp/DefaultLogger.hpp>
+#include <assimp/scene.h>
+#include <assimp/importerdesc.h>
 
 using namespace Assimp;
 
@@ -99,7 +100,7 @@ const aiImporterDesc* RAWImporter::GetInfo () const
 void RAWImporter::InternReadFile( const std::string& pFile,
     aiScene* pScene, IOSystem* pIOHandler)
 {
-    boost::scoped_ptr<IOStream> file( pIOHandler->Open( pFile, "rb"));
+    std::unique_ptr<IOStream> file( pIOHandler->Open( pFile, "rb"));
 
     // Check whether we can read from the file
     if( file.get() == NULL) {
@@ -183,12 +184,11 @@ void RAWImporter::InternReadFile( const std::string& pFile,
             }
 
             // search in the list of meshes whether we have one with this texture
-            for (std::vector< MeshInformation >::iterator it = (*curGroup).meshes.begin(),
-                end = (*curGroup).meshes.end(); it != end; ++it)
+            for (auto &mesh : (*curGroup).meshes)
             {
-                if (length == (*it).name.length() && (length ? !::strcmp(sz,(*it).name.c_str()) : true))
+                if (length == mesh.name.length() && (length ? !::strcmp(sz, mesh.name.c_str()) : true))
                 {
-                    output = &(*it);
+                    output = &mesh;
                     break;
                 }
             }
@@ -223,13 +223,12 @@ void RAWImporter::InternReadFile( const std::string& pFile,
 
     // count the number of valid groups
     // (meshes can't be empty)
-    for (std::vector< GroupInformation >::iterator it = outGroups.begin(), end = outGroups.end();
-        it != end;++it)
+    for (auto & outGroup : outGroups)
     {
-        if (!(*it).meshes.empty())
+        if (!outGroup.meshes.empty())
         {
             ++pScene->mRootNode->mNumChildren;
-            pScene->mNumMeshes += (unsigned int)(*it).meshes.size();
+            pScene->mNumMeshes += (unsigned int) outGroup.meshes.size();
         }
     }
 
@@ -251,10 +250,9 @@ void RAWImporter::InternReadFile( const std::string& pFile,
     aiMaterial** mats = pScene->mMaterials = new aiMaterial*[pScene->mNumMaterials];
 
     unsigned int meshIdx = 0;
-    for (std::vector< GroupInformation >::iterator it = outGroups.begin(), end = outGroups.end();
-        it != end;++it)
+    for (auto & outGroup : outGroups)
     {
-        if ((*it).meshes.empty())continue;
+        if (outGroup.meshes.empty())continue;
 
         aiNode* node;
         if (pScene->mRootNode->mNumChildren)
@@ -263,13 +261,13 @@ void RAWImporter::InternReadFile( const std::string& pFile,
             node->mParent = pScene->mRootNode;
         }
         else node = *cc;++cc;
-        node->mName.Set((*it).name);
+        node->mName.Set(outGroup.name);
 
         // add all meshes
-        node->mNumMeshes = (unsigned int)(*it).meshes.size();
+        node->mNumMeshes = (unsigned int) outGroup.meshes.size();
         unsigned int* pi = node->mMeshes = new unsigned int[ node->mNumMeshes ];
-        for (std::vector< MeshInformation >::iterator it2 = (*it).meshes.begin(),
-            end2 = (*it).meshes.end(); it2 != end2; ++it2)
+        for (std::vector< MeshInformation >::iterator it2 = outGroup.meshes.begin(),
+            end2 = outGroup.meshes.end(); it2 != end2; ++it2)
         {
             ai_assert(!(*it2).vertices.empty());
 
